:- lib(ic).
:- lib(ic_search).

%-----------
% bit-vector type 
% X is the word/array identifier
% L is the length
bit_vector_t(X,L) :- dim(X,[L]), term_variables(X, Xlist), Xlist :: [0,1].

% dim(?Term, ?Dimensions)
% Creates a multi-dimensional array in the form of nested structures, or 
% computes the dimensions of an existing matrix. 

% term_variables(?Term, -VarList)
% This predicate collects all the variables inside Term into the list VarList.

% ?Vars #:: ++Domain
% Constrain Vars to be integral and have the domain Domain. 
%-----------

% integer type 
% X is the identifier
% L is the left bound
% R is the right bound
integer_t(X,L,R) :- X :: L .. R.

%----------- 		
search_func(VAR) :- search(VAR,0,input_order,indomain_random,complete,[]).
% search(+L, ++Arg, ++Select, +Choice, ++Method, +Option)
% A generic search routine for finite domains or IC which implements different 
% partial search methods (complete, credit, lds, bbs, dbs, sbds, gap_sbds, 
% gap_sbdd)
%
% - L is a collection of domain variables (Arg = 0) or a collection of terms 
%     (Arg > 0) 
% - Arg is an integer, which is 0 if the list is a list of dvarints or greater 
%     0 if the list consists of terms of arity greater than Arg, the value Arg 
%     indicates the selected argument of the term 
%	- Select is a predefined selection method or the name of a predicate of arity 
%     2. Predefined methods are input_order, first_fail, smallest, largest, 
%     occurrence, most_constrained, max_regret, anti_first_fail 
% - Choice is the name of a predicate of arity 1 or a term with two arguments 
%     with the same functor as a predicate of arity 3. Some names are already 
%     predefined as special cases and are handled without a meta-call: indomain,
%     indomain_min, indomain_max, indomain_middle, indomain_median, 
%     indomain_split, indomain_random, indomain_interval 
% - Method is one of the following: complete, bbs(Steps:integer), 
%     lds(Disc:integer), credit(Credit:integer, Extra:integer or bbs
%     (Steps:integer) or lds(Disc:integer)), dbs(Level:integer, Extra:integer or 
%     bbs(Steps:integer) or lds(Disc:integer)), sbds, gap_sbds, gap_sbdd 
% - Option is a list of option terms. Currently recognized are backtrack(-N), 
%     node(++Call), nodes(++N) 
%-----------

%-----------
search_int_func(VAR) :- indomain(VAR,random).
% indomain(?Var, ++Method)
% a flexible way to assign values to finite domain variables
% - Var
%     a domain variable or an integer 
% - Method
%     one of the atoms min, max, middle, median, split, interval, random or an 
%     integer 
%-----------

%-----------
search_bv_func( VAR ) :- VAR =.. [_|Var_VAR], search_func( Var_VAR ).
%-----------

%-----------
nth( VAR, I, VAR_I ) :- arg(I, VAR, VAR_I).
% arg(+N, +Term, -Arg)
% Succeeds if Arg is the Nth argument of the compound term Term. 
%-----------

%-----------
print_obj( Obj ) :- dim( Obj, [N] ), ( for( J, 1, N ), param( Obj ) do Ob is Obj[J], printf( "%1d", Ob ) ), nl.
%-----------

get_obj( Obj ) :- term_string( Obj, String ), writeln( String ). 


% X[1..8], Y[1..8], X[2..3] == "11" & Y[1] == '1' & Y[1] == X[4] 

%constraint_1 :- (X_SIZE #= 8),
%                (Y_SIZE #= 8),
%                bit_vector_t(X,X_SIZE),
%                bit_vector_t(Y,Y_SIZE),
%                (X[2] #= 1) and (X[3] #= 1) and (Y[1] #= 1) and (Y[1] #= X[4]),
%                search_bv_func( Var_X ), %X =.. [_|Var_X], search_func(Var_X),
%                search_bv_func( Var_Y ), %Y =.. [_|Var_Y], search_func(Var_Y),
%                write("X = "), writeln(X),
%                write("Y = "), writeln(Y).
%
%% X[1..8], Y[1..8], X[2..3] == "11" & Y[1] == '1' & Y[1] == X[4] and V == 0 and U > 0
%
%constraint_2 :- VECT_LENGTH is 8,
%             L_BOUND is -(2^7),
%             R_BOUND is (2^7)-1,
%             integer_t( U, L_BOUND, R_BOUND ),
%             integer_t( V, L_BOUND, R_BOUND ),
%             bit_vector_t( X, VECT_LENGTH ),
%             bit_vector_t( Y, VECT_LENGTH ),
%             write( "U : (" ), write( L_BOUND ), write( ", " ), write( R_BOUND ), writeln( ")" ),
%             write( "V : (" ), write( L_BOUND ), write( ", " ), write( R_BOUND ), writeln( ")" ),
%             write( "X : [1.." ), write( VECT_LENGTH ), writeln( "]" ),
%             write( "Y : [1.." ), write( VECT_LENGTH ), writeln( "]" ),
%             (Y[1] #= 1) and (Y[1] #= X[4]) and ( V #= 0 ) and ( U #> 0),
%             arg( 1, X, X_1 ), X_1 #= 1,
%             arg( 2, X, X_2 ), X_2 #= 1,
%             arg( 3, X, X_3 ), X_3 #= 1,
%             arg( 4, X, X_4 ), X_4 #= 1,
%             arg( 5, X, X_5 ), X_5 #= 1,
%             arg( 6, X, X_6 ), X_6 #= 1,
%             arg( 7, X, X_7 ), X_7 #= 1,
%             arg( 8, X, X_8 ), X_8 #= 1,
%             search_int_func( U ),
%             search_int_func( V ),
%             search_bv_func( X ),
%             search_bv_func( Y ),
%             write("V = "), writeln( V ),
%             write("U = "), writeln( U ),
%             write("X = "), print_obj( X ),
%             write("Y = "), print_obj( Y ).
%
%constraint_3 :- (X_SIZE #= 8),
%                bit_vector_t(X,X_SIZE),
%                (X[2] #= 1),
%                search_bv_func( Var_X ),
%                search_bv_func( Var_Y ),
%                write("X = "), writeln(X),
%                write("Y = "), writeln(Y).
%
%
